//
//  ViewController.swift
//  Smoky
//
//  Created by Mark Meretzky on 12/15/14.
//  Copyright (c) 2014 Mark Meretzky. All rights reserved.
//

import UIKit;

class ViewController: UIViewController, UITextViewDelegate {
	let editedFile: String;	//name of the file that holds the text edited by the user
	let dateFormatter: NSDateFormatter = NSDateFormatter();
	let timeFormatter: NSDateFormatter = NSDateFormatter();

	override init() {
		//Get the name of this app's Documents directory.
		let directories: [AnyObject] = NSSearchPathForDirectoriesInDomains(
			NSSearchPathDirectory.DocumentDirectory,
			NSSearchPathDomainMask.UserDomainMask,
			true);
		assert(directories.count > 0);
		let documentsDirectory: String = directories[0] as String;

		//Create the full pathname of the edited file.  This file does not exist yet.
		editedFile = documentsDirectory.stringByAppendingPathComponent("edited.txt");

		super.init(nibName: nil, bundle: nil);
		navigationItem.rightBarButtonItem = editButtonItem();

		//When the keyboard becomes visible, give the UITextView a bottom margin
		//as tall as the keyboard.  This will prevent the keyboard from hiding
		//any of the text in the file.
		let notificationCenter: NSNotificationCenter = NSNotificationCenter.defaultCenter();

		notificationCenter.addObserver(
			self,
			selector: "keyboardWillShow:",
			name: UIKeyboardWillShowNotification,
			object: nil);

		notificationCenter.addObserver(
			self,
			selector: "keyboardWillHide:",
			name: UIKeyboardWillHideNotification,
			object: nil);

		//Display the date and time whenever we finish editing the file.
		dateFormatter.dateStyle = NSDateFormatterStyle.FullStyle;
		dateFormatter.timeStyle = NSDateFormatterStyle.NoStyle;
		timeFormatter.dateStyle = NSDateFormatterStyle.NoStyle;
		timeFormatter.timeStyle = NSDateFormatterStyle.FullStyle;
	}

	//Never called.
	required init(coder aDecoder: NSCoder) {
		editedFile = "";
		super.init(coder: aDecoder);
	}

	override func loadView() {
		let textView: UITextView = UITextView();
		textView.font = UIFont(name: "Helvetica", size: 17);
		textView.keyboardType = UIKeyboardType.Default;
		textView.returnKeyType = UIReturnKeyType.Default;
		textView.spellCheckingType = UITextSpellCheckingType.Default;
		textView.autocapitalizationType = UITextAutocapitalizationType.Sentences;
		textView.editable = false;    //Don't pop up a keyboard when touched.
		textView.scrollsToTop = true; //when you tap the status bar
		textView.bounces = true;      //against floor or ceiling
		textView.decelerationRate = UIScrollViewDecelerationRateNormal; //friction
		textView.delegate = self;

		//If the edited file exists, open it.
		//Otherwise, open the unedited file.

		let fileManager: NSFileManager = NSFileManager.defaultManager();
		var file: String?;
		if fileManager.fileExistsAtPath(editedFile, isDirectory: nil) {
			file = editedFile;
		} else {
			let bundle: NSBundle = NSBundle.mainBundle();
			let uneditedFile: String? = bundle.pathForResource("unedited", ofType: "txt");
			if uneditedFile == nil {
				println("could not find file unedited.txt");
			} else {
				file = uneditedFile!;
			}
		}

		if file != nil {
			var error: NSError?;
			let s: String? = String(contentsOfFile: file!, encoding: NSUTF8StringEncoding, error: &error);
			if s == nil {
				println("could not read \(file!):\n\(error!)");
			} else {
				textView.text = s!;
				//When we press the Edit button, put the blinking insertion point at the start of the file.
				//Can't give a value to .selectedRange until after we give a value to .text.
				textView.selectedRange = NSMakeRange(0, 0);
			}
		}

		view = textView;
	}

	override func viewDidLoad() {
		super.viewDidLoad()
		// Do any additional setup after loading the view, typically from a nib.
	}

	//This method is called by the Edit/Done button.
	
	override func setEditing(editing: Bool, animated: Bool) {
		super.setEditing(editing, animated: animated);
		let textView: UITextView = view as UITextView;
		textView.editable = editing;
		if editing {
			//Display the keyboard, even if the UITextView hasn't been touched yet.
			textView.becomeFirstResponder();
		} else {
			//The next time we edit, the blinking insertion point will be at the start of the file.
			textView.selectedRange = NSMakeRange(0, 0);
		}
	}

	//This method of the UITextFieldDelegate is called
	//when the user presses the done button.

	func textViewShouldEndEditing(textView: UITextView) -> Bool {
		//true means we should hide the keyboard and call textViewDidEndEditing.
		return true;
	}

	//This method of the UITextFieldDelegate is called
	//when textViewShouldEndEditing returns true.

	func textViewDidEndEditing(textView: UITextView) {
		var error: NSError?;

		let success: Bool = textView.text.writeToFile(
			editedFile,
			atomically: true,
			encoding: NSUTF8StringEncoding,
			error: &error);

		//Display a modal view to tell if the file was written successfully.

		let modalViewController: UIViewController = UIViewController(nibName: nil, bundle: nil);

		modalViewController.navigationItem.rightBarButtonItem = UIBarButtonItem(
			title: "OK",
			style: UIBarButtonItemStyle.Done,
			target: self,
			action: "ok");

		let v: UITextView = UITextView();
		v.backgroundColor = UIColor.orangeColor();
		v.font = (view as UITextView).font;	//same font as that of this ViewController's UITextView.
		v.editable = false;

		if !success {
			v.text = "could not write to file \(editedFile): \(error!)"
		} else {
			let manager: NSFileManager = NSFileManager.defaultManager();
			let attributes: [NSObject: AnyObject]? = manager.attributesOfItemAtPath(editedFile, error: &error);
			if attributes == nil {
				v.text = "could not get attributes for file \(editedFile): \(error!)";
			} else {
				v.text = "\(attributes![NSFileSize]!) byte(s) written\n\n";
				let date: NSDate? = attributes![NSFileModificationDate]? as NSDate?;
				if date != nil {
					v.text = v.text + "on \(dateFormatter.stringFromDate(date!))\n\n"
						+ "at \(timeFormatter.stringFromDate(date!))\n\n"
				}

				let permissions: Int? = attributes![NSFilePosixPermissions] as Int?;
				if permissions != nil {
					let octal: String = String(format: "%3o", permissions!);
					v.text = v.text + "with permissions \(octal)\n\n";
				}

				v.text = v.text + "to file \"\(editedFile.lastPathComponent)\"\n\n";

				let name: String? = attributes![NSFileOwnerAccountName] as String?;
				if name != nil {
					v.text = v.text + "owned by user \"\(name!)\"\n\n";
				}

				v.text = v.text + "in directory\n\n\(editedFile.stringByDeletingLastPathComponent)";
			}
		}

		modalViewController.view = v;
		let navigationController: UINavigationController =
			UINavigationController(rootViewController: modalViewController);
		presentViewController(navigationController, animated: true, completion: nil);
	}

	//Called when the user presses the OK button.
	//Dismiss the modal view controller.

	func ok() {
		dismissViewControllerAnimated(true, completion: nil);
	}

	//Called by the notification center when the keyboard appears.
	//Give the UITextView a bottom margin as tall as the keyboard.
	//This prevents the keyboard from covering any text.

	func keyboardWillShow(notification: NSNotification) {
		let userInfo: [NSObject: AnyObject]? = notification.userInfo;	//an optional Dictionary
		if userInfo != nil {
			let value: NSValue? = userInfo![UIKeyboardFrameBeginUserInfoKey] as NSValue?;
			if value != nil {
				let rect: CGRect = value!.CGRectValue();
				//top, left, bottom, right
				let edgeInsets: UIEdgeInsets = UIEdgeInsetsMake(topLayoutGuide.length, 0, rect.size.height, 0);
				let textView: UITextView = view as UITextView;
				textView.contentInset = edgeInsets;
				textView.scrollIndicatorInsets = edgeInsets;

			}
		}
	}

	//Called by the notification center when the keyboard disappears.
	//Remove the UITextView's bottom margin.

	func keyboardWillHide(notification: NSNotification) {
		let userInfo: [NSObject: AnyObject]? = notification.userInfo;	//an optional Dictionary
		if userInfo != nil {
			let value: NSValue? = userInfo![UIKeyboardFrameEndUserInfoKey] as NSValue?;
			if value != nil {
				let rect: CGRect = value!.CGRectValue();
				let textView: UITextView = view as UITextView;
				let edgeInsets: UIEdgeInsets = UIEdgeInsetsMake(topLayoutGuide.length, 0, 0, 0);
				textView.contentInset = edgeInsets;
				textView.scrollIndicatorInsets = edgeInsets;
			}
		}
	}

	override func didReceiveMemoryWarning() {
		super.didReceiveMemoryWarning()
		// Dispose of any resources that can be recreated.
	}

}
